package com.qianhe.system.service.impl;

import com.qianhe.common.exception.ServiceException;
import com.qianhe.common.utils.DateUtils;
import com.qianhe.common.utils.StringUtils;
import com.qianhe.system.domain.WaterGoods;
import com.qianhe.system.domain.WaterGoodsImg;
import com.qianhe.system.domain.WaterOrder;
import com.qianhe.system.domain.WaterOrderGoods;
import com.qianhe.system.mapper.WaterGoodsImgMapper;
import com.qianhe.system.mapper.WaterGoodsMapper;
import com.qianhe.system.mapper.WaterOrderGoodsMapper;
import com.qianhe.system.mapper.WaterOrderMapper;
import com.qianhe.system.service.IWaterOrderService;
import com.qianhe.system.vo.WaterOrderVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Random;

/**
 * 订单Service业务层处理
 *
 * @author qianhe
 * @date 2023-11-23
 */
@Service
public class WaterOrderServiceImpl implements IWaterOrderService
{
    @Autowired
    private WaterOrderMapper waterOrderMapper;
    @Autowired
    private WaterOrderGoodsMapper waterOrderGoodsMapper;
    @Autowired
    private WaterGoodsMapper waterGoodsMapper;
    @Autowired
    private WaterGoodsImgMapper waterGoodsImgMapper;

    /**
     * 查询订单
     *
     * @param id 订单主键
     * @return 订单
     */
    @Override
    public WaterOrderVo selectWaterOrderById(Long id)
    {
        WaterOrder waterOrder = waterOrderMapper.selectWaterOrderById(id);
        //转vo
        WaterOrderVo waterOrderVo = new WaterOrderVo();
        BeanUtils.copyProperties(waterOrder,waterOrderVo);
        //查询该订单商品
        WaterOrderGoods waterOrderGoods = new WaterOrderGoods();
        waterOrderGoods.setOrderId(id);
        List<WaterOrderGoods> waterOrderGoodsList = waterOrderGoodsMapper.selectWaterOrderGoodsList(waterOrderGoods);
        //查询商品图片信息
        List<WaterGoodsImg> waterGoodsImgs = waterGoodsImgMapper.selectWaterGoodsImgList(new WaterGoodsImg());
        //订单商品集合
        List<WaterOrderGoods> waterOrderGoodsList1 = new ArrayList<>();
        if (waterOrderGoodsList.size() > 0){
            for (WaterOrderGoods waterOrderGood : waterOrderGoodsList) {
                //封面图集合
                List<WaterGoodsImg> coverImgs = new ArrayList<>();
                //详情图集合
                List<WaterGoodsImg> detailsImgs = new ArrayList<>();
                for (WaterGoodsImg waterGoodsImg : waterGoodsImgs) {
                    if (waterOrderGood.getGoodsId().equals(waterGoodsImg.getGoodsId()) && waterGoodsImg.getImgType() == 1){
                        coverImgs.add(waterGoodsImg);
                    }else if (waterOrderGood.getGoodsId().equals(waterGoodsImg.getGoodsId()) && waterGoodsImg.getImgType() == 2){
                        detailsImgs.add(waterGoodsImg);
                    }
                }
                if (coverImgs.size() > 0){
                    waterOrderGood.setCoverImgs(coverImgs);
                }
                if (detailsImgs.size() > 0){
                    waterOrderGood.setDetailsImgs(detailsImgs);
                }
                waterOrderGoodsList1.add(waterOrderGood);
            }
        }
        waterOrderVo.setWaterOrderGoodsList(waterOrderGoodsList1);
        return waterOrderVo;
    }

    /**
     * 查询订单列表
     *
     * @param waterOrderVo 订单
     * @return 订单
     */
    @Override
    public List<WaterOrder> selectWaterOrderList(WaterOrderVo waterOrderVo)
    {
        return waterOrderMapper.selectWaterOrderList(waterOrderVo);
    }

    /**
     * 新增订单
     *
     * @param waterOrderVo 订单
     * @return 结果
     */
    @Override
    @Transactional
    public int insertWaterOrder(WaterOrderVo waterOrderVo)
    {
        waterOrderVo.setCreateUser(waterOrderVo.getUserId().toString());
        waterOrderVo.setCreateTime(DateUtils.getNowDate());
        waterOrderVo.setOrderNum(getOrerNum());
        waterOrderVo.setOrderState(1);
        waterOrderVo.setPayState(0);
        waterOrderVo.setOrderType(1);
        int i = waterOrderMapper.insertWaterOrder(waterOrderVo);
        //新增订单商品
        insertWaterOrderGoods(waterOrderVo);
        return i;
    }

    /**
     * 生成订单编号
     * @return
     */
    private String getOrerNum(){
        String orderNum = "";
        //获取当前年月日
        SimpleDateFormat format = new SimpleDateFormat("yyyyMMdd");
        String today = format.format(DateUtils.getNowDate());
        Long rqbh = Long.parseLong(today + "0001");
        while (true){
            WaterOrder waterOrder = new WaterOrder();
            waterOrder.setOrderNum(rqbh.toString());
            WaterOrder waterOrder1 = waterOrderMapper.selectWaterOrderNumByOrderNum(rqbh.toString());
            if (StringUtils.isNotNull(waterOrder1)){
                rqbh ++;
            }else {
                break;
            }
        }
        //生成随机四位英文字母
        Random random = new Random();
        String randomletters = "";
        for (int i = 0; i < 4; i++) {
            int index = random.nextInt(26);
            char letter = (char) ('A' + index);
            randomletters += letter;
        }
        orderNum = rqbh + randomletters;
        return orderNum;
    }

    private void insertWaterOrderGoods(WaterOrderVo waterOrderVo) {
        List<WaterOrderGoods> waterOrderGoodsList = waterOrderVo.getWaterOrderGoodsList();
        //商品订单主键id
        Long id = waterOrderVo.getId();
        //商品订单编号
        String orderNum = waterOrderVo.getOrderNum();
        if (StringUtils.isNotNull(waterOrderGoodsList)){
            for (WaterOrderGoods waterOrderGoods : waterOrderGoodsList) {
                //根据商品id查询商品信息
                WaterGoods waterGoods = waterGoodsMapper.selectWaterGoodsById(waterOrderGoods.getGoodsId());
                waterOrderGoods.setOrderId(id);
                waterOrderGoods.setOrderNum(orderNum);
                waterOrderGoods.setGoodsTypeId(waterGoods.getGoodsTypeId());
                waterOrderGoods.setGoodsTitle(waterGoods.getTitle());
                waterOrderGoods.setCreateUser(waterOrderVo.getUserId().toString());
                waterOrderGoods.setCreateTime(DateUtils.getNowDate());
            }
            //批量新增订单商品
            waterOrderMapper.batchInsertWaterOrderGoods(waterOrderGoodsList);
        }
    }

    /**
     * 修改订单
     *
     * @param waterOrderVo 订单
     * @return 结果
     */
    @Override
    @Transactional
    public int updateWaterOrder(WaterOrderVo waterOrderVo)
    {
        //删除订单商品
        waterOrderMapper.deleteWaterOrderGoodsByOrderId(waterOrderVo.getId());
        //新增订单商品
        insertWaterOrderGoods(waterOrderVo);
        return waterOrderMapper.updateWaterOrder(waterOrderVo);
    }

    /**
     * 批量删除订单
     *
     * @param ids 需要删除的订单主键
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteWaterOrderByIds(Long[] ids)
    {
        //批量删除订单商品
        waterOrderMapper.deleteWaterOrderGoodsByOrderIds(ids);
        return waterOrderMapper.deleteWaterOrderByIds(ids);
    }

    /**
     * 删除订单信息
     *
     * @param id 订单主键
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteWaterOrderById(Long id)
    {
        return waterOrderMapper.deleteWaterOrderById(id);
    }

    /**
     * 修改订单状态
     * @param waterOrderVo
     * @return
     */
    @Override
    @Transactional
    public int updateOrderState(WaterOrderVo waterOrderVo) {
        //接单时需再次查看该订单是否已经被用户所取消
        if (waterOrderVo.getOrderState() == 3){
            WaterOrder waterOrder = waterOrderMapper.selectWaterOrderById(waterOrderVo.getId());
            if (waterOrder.getOrderState() == 5){
                throw new ServiceException("该订单已被用户取消！");
            }
        }
        return waterOrderMapper.updateWaterOrder(waterOrderVo);
    }

    /**
     * 统计站点订单数量
     * @param stationId
     * @return
     */
    @Override
    public Map<String, Integer> getOrderNumByStation(Long stationId) {
        WaterOrderVo waterOrderVo = new WaterOrderVo();
        waterOrderVo.setStationId(stationId);
        waterOrderVo.setCreateTime(DateUtils.getNowDate());
        return waterOrderMapper.getOrderNumByStation(waterOrderVo);
    }

    /**
     * 根据站点查询订单信息
     * @param waterOrderVo 订单
     * @return
     */
    @Override
    public List<WaterOrder> getWaterOrderListByStationId(WaterOrderVo waterOrderVo) {
        return waterOrderMapper.getWaterOrderListByStationId(waterOrderVo);
    }

    /**
     * 修改订单完成状态（送水端）
     * @param waterOrderVo
     * @return
     */
    @Override
    public int updateCompleteState(WaterOrderVo waterOrderVo) {
        return waterOrderMapper.updateWaterOrder(waterOrderVo);
    }

    /**
     * 修改订单确认状态（订水端）
     * @param waterOrderVo
     * @return
     */
    @Override
    public int updateConfirmState(WaterOrderVo waterOrderVo) {
        return waterOrderMapper.updateWaterOrder(waterOrderVo);
    }
}
